using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Workflow.ComponentModel;
using System.Workflow.Activities.Rules;
using System.Workflow.ComponentModel.Compiler;
using System.Reflection;
using System.Collections;
using System.Workflow.Activities.Rules.Design;

namespace JonKruger.EnhancedRuleSetDialog
{
    public partial class EnhancedRuleSetDialog : System.Workflow.Activities.Rules.Design.RuleSetDialog
    {
        private MemberListSettingsType _memberListSettings = MemberListSettingsType.Default;
        private bool _includeStaticMembers = true;
        private bool _manuallyFiringPopulateAutoCompleteList = false;

        /// <summary>
        /// Indicates whether only public members will be shown in the 
        /// Intellisense in the dialog textboxes.
        /// </summary>
        [DefaultValue(MemberListSettingsType.Default)]
        [Description("Indicates how the list of members will be filtered in the Intellisense in the dialog text boxes.")]
        public MemberListSettingsType MemberListSettings
        {
            get { return _memberListSettings; }
            set { _memberListSettings = value; }
        }

        /// <summary>
        /// Indicates whether static members will be included in the Intellisense in the dialog text boxes.
        /// </summary>
        [DefaultValue(true)]
        [Description("Indicates whether static members will be included in the Intellisense in the dialog text boxes.")]
        public bool IncludeStaticMembers
        {
            get { return _includeStaticMembers; }
            set { _includeStaticMembers = value; }
        }
	
        /// <summary>
        /// Handle this event if you want to modify the list of members
        /// in the Intellisense.
        /// </summary>
        public event EventHandler<IntellisenseEventArgs> ModifyIntellisenseList;

        public EnhancedRuleSetDialog(Activity activity, RuleSet ruleSet) : base(activity, ruleSet)
        {
            HandleIntellisenseEvents();
            _includeStaticMembers = true;
        }

        public EnhancedRuleSetDialog(Type activityType, ITypeProvider typeProvider, RuleSet ruleSet) : base(activityType, typeProvider, ruleSet)
        {
            HandleIntellisenseEvents();
            _includeStaticMembers = true;
        }

        protected virtual void OnModifyIntellisenseList(IntellisenseEventArgs e)
        {
            EventHandler<IntellisenseEventArgs> handler = ModifyIntellisenseList;
            if (handler != null)
                handler.Invoke(this, e);
        }

        private void HandleIntellisenseEvents()
        {
            // Get the three text box objects
            object conditionTextBox = typeof(RuleSetDialog).GetField("conditionTextBox", BindingFlags.NonPublic | BindingFlags.Instance).GetValue(this);
            object thenTextBox = typeof(RuleSetDialog).GetField("thenTextBox", BindingFlags.NonPublic | BindingFlags.Instance).GetValue(this);
            object elseTextBox = typeof(RuleSetDialog).GetField("elseTextBox", BindingFlags.NonPublic | BindingFlags.Instance).GetValue(this);

            // Get the Type object that we will use to create an 
            // EventHandler<AutoCompletionEventArgs> object
            Type handlerType = Type.GetType("System.EventHandler`1").MakeGenericType(Type.GetType("System.Workflow.Activities.Rules.Design.AutoCompletionEventArgs, System.Workflow.Activities, Version=3.0.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"));

            // Get the method that will handle the event
            MethodInfo method = GetType().GetMethod("PopulateAutoCompleteList", BindingFlags.NonPublic | BindingFlags.Instance);

            // Create our EventHandler<AutoCompletionEventArgs> object
            Delegate handler = Delegate.CreateDelegate(handlerType, this, method);

            // Handle the text box events
            conditionTextBox.GetType().GetEvent("PopulateAutoCompleteList").AddEventHandler(conditionTextBox, handler);
            thenTextBox.GetType().GetEvent("PopulateAutoCompleteList").AddEventHandler(thenTextBox, handler);
            elseTextBox.GetType().GetEvent("PopulateAutoCompleteList").AddEventHandler(elseTextBox, handler);
        }

        private void PopulateAutoCompleteList(object sender, EventArgs e)
        {
            RuleSetDialogTextBoxType textBoxType;
            if (((Control) sender).Name == "conditionTextBox")
                textBoxType = RuleSetDialogTextBoxType.Condition;
            else if (((Control) sender).Name == "thenTextBox")
                textBoxType = RuleSetDialogTextBoxType.Then;
            else if (((Control) sender).Name == "elseTextBox")
                textBoxType = RuleSetDialogTextBoxType.Else;
            else
                throw new InvalidOperationException("Unknown text box.");

            PropertyInfo autoCompleteValuesProperty = e.GetType().GetProperty("AutoCompleteValues");
            ICollection collection = (ICollection)autoCompleteValuesProperty.GetValue(e, null);

            if (this.MemberListSettings == MemberListSettingsType.PublicOnly)
            {
                ArrayList newList = new ArrayList();
                foreach (object o in collection)
                {
                    if (o is MethodInfo)
                    {
                        MethodInfo info = (MethodInfo)o;
                        if (info.IsPublic)
                            newList.Add(o);
                    }
                    else if (o is PropertyInfo)
                    {
                        PropertyInfo info = (PropertyInfo)o;
                        MethodInfo getter = info.GetGetMethod();
                        if (getter != null && getter.IsPublic)
                            newList.Add(o);
                        else
                        {
                            MethodInfo setter = info.GetSetMethod();
                            if (setter != null && setter.IsPublic)
                                newList.Add(o);
                        }
                    }
                    else if (o is FieldInfo)
                    {
                        FieldInfo info = (FieldInfo)o;
                        if (info.IsPublic)
                            newList.Add(o);
                    }
                    else
                        newList.Add(o);
                }
                collection = newList;
            }
            else if (this.MemberListSettings == MemberListSettingsType.AttributeOnly)
            {
                ArrayList newList = new ArrayList();
                foreach (object o in collection)
                {
                    if (o is MethodInfo)
                    {
                        foreach (IncludeInRuleSetDialogAttribute attr in ((MethodInfo)o).GetCustomAttributes(typeof(IncludeInRuleSetDialogAttribute), true))
                        {
                            if ((attr.TextBoxTypes & textBoxType) == textBoxType)
                                newList.Add(o);
                        }
                    }
                    else if (o is PropertyInfo)
                    {
                        foreach (IncludeInRuleSetDialogAttribute attr in ((PropertyInfo)o).GetCustomAttributes(typeof(IncludeInRuleSetDialogAttribute), true))
                        {
                            if ((attr.TextBoxTypes & textBoxType) == textBoxType)
                                newList.Add(o);
                        }
                    }
                    else if (o is FieldInfo)
                    {
                        foreach (IncludeInRuleSetDialogAttribute attr in ((FieldInfo)o).GetCustomAttributes(typeof(IncludeInRuleSetDialogAttribute), true))
                        {
                            if ((attr.TextBoxTypes & textBoxType) == textBoxType)
                                newList.Add(o);
                        }
                    }
                    else
                        newList.Add(o);
                }
                collection = newList;
            }

            if (!this.IncludeStaticMembers)
            {
                ArrayList newList = new ArrayList();
                foreach (object o in collection)
                {
                    if (!(o is Type))
                        newList.Add(o);
                }
                collection = newList;
            }

            IntellisenseEventArgs args = new IntellisenseEventArgs(collection, textBoxType);
            OnModifyIntellisenseList(args);
            collection = args.IntellisenseList;

            autoCompleteValuesProperty.SetValue(e, collection, null);
        }
    }
}